; **********************************************************************
; *                                                                    *
; * Author   : Daniel Brall                                            *
; * Website  : http://www.bradan.eu/                                   *
; * E-Mail   : vectorizer@bradan.eu                                    *
; * Copyright: Daniel Brall                                            *
; *            All rights reserved!                                    *
; *                                                                    *
; * You are not allowed to modify and/or redistribute this sourcecode! *
; *                                                                    *
; **********************************************************************

XIncludeFile "vectorizer.pbi"
XIncludeFile "scanline.pbi"

NewList PathList.SPath()

Procedure CreateImageFromPaths(List Paths.SPath(), image.i, offsetX.d, offsetY.d, scaleX.d, scaleY.d)
  Protected line.SLine
  Protected colorFactor.d
  
  StartDrawing(ImageOutput(image))
  DrawingMode(#PB_2DDrawing_AllChannels)
  Box(0, 0, ImageWidth(image), ImageHeight(image), RGBA(255, 255, 255, 0))
  
  ForEach Paths()
    DrawPolygon(Paths()\Path(), offsetX, offsetY, scaleX, scaleY, Paths()\avgColor)
  Next
  
  ForEach Paths()
    If ListSize(Paths()\Path()) > 0
      colorFactor = 255.0 / ListSize(Paths()\Path())
      
      FirstElement(Paths()\Path())
      CopyStructure(@Paths()\Path(), @line\a, SVector)
      CopyStructure(@Paths()\Path(), @line\b, SVector)
      
      ForEach Paths()\Path()
        CopyStructure(@line\b, @line\a, SVector)
        CopyStructure(@Paths()\Path(), @line\b, SVector)
        
        LineXY(line\a\x * scaleX + offsetX, line\a\y * scaleY + offsetY, line\b\x * scaleX + offsetX, line\b\y * scaleY + offsetY, RGBA(ListIndex(Paths()\Path()) * colorFactor, 0, 0, 255))
      Next
    EndIf
  Next
  StopDrawing()
EndProcedure

Procedure CreateSVGFromPaths(filename.s, List Paths.SPath(), width.i, height.i, offsetX.d, offsetY.d, scaleX.d, scaleY.d)
  Protected file.i
  Protected points.s
  
  file = CreateFile(#PB_Any, filename)
  If file
    WriteStringN(file, "<?xml version="+#DQUOTE$+"1.0"+#DQUOTE$+" encoding="+#DQUOTE$+"UTF-8"+#DQUOTE$+"?>")
    WriteStringN(file, "<svg xmlns="+#DQUOTE$+"http://www.w3.org/2000/svg"+#DQUOTE$)
    WriteStringN(file, "	xmlns:xlink="+#DQUOTE$+"http://www.w3.org/1999/xlink"+#DQUOTE$)
    WriteStringN(file, "	xmlns:ev="+#DQUOTE$+"http://www.w3.org/2001/xml-events"+#DQUOTE$)
    WriteStringN(file, "	version="+#DQUOTE$+"1.1"+#DQUOTE$+" baseProfile="+#DQUOTE$+"full"+#DQUOTE$)
    WriteStringN(file, "	width="+#DQUOTE$+Str(width)+"px"+#DQUOTE$+" height="+#DQUOTE$+Str(height)+"px"+#DQUOTE$+" viewBox="+#DQUOTE$+"0 0 "+Str(width)+" "+Str(height)+#DQUOTE$+">")
    
    ForEach Paths()
      points = ""
      ForEach Paths()\Path()
        points + Str(Paths()\Path()\x) + " " + Str(Paths()\Path()\y) + " "
      Next
      WriteStringN(file, "	<polygon points="+#DQUOTE$+RTrim(points)+#DQUOTE$+" fill="+#DQUOTE$+"#"+RSet(Hex(Red(Paths()\avgColor)), 2, "0")+RSet(Hex(Green(Paths()\avgColor)), 2, "0")+RSet(Hex(Blue(Paths()\avgColor)), 2, "0")+#DQUOTE$+"/>")
    Next
    WriteStringN(file, "</svg>")
    CloseFile(file)
  EndIf
EndProcedure

Procedure ViewImage(Image.i)
  Protected window.i
  
  If IsImage(Image)
    window = OpenWindow(#PB_Any, #PB_Ignore, #PB_Ignore, ImageWidth(Image), ImageHeight(Image), "Image", #PB_Window_TitleBar | #PB_Window_SystemMenu)
    
    ImageGadget(#PB_Any, 0, 0, ImageWidth(Image), ImageHeight(Image), ImageID(Image))
    
    Repeat
    Until WaitWindowEvent(5) = #PB_Event_CloseWindow
    
    CloseWindow(window)
  EndIf
EndProcedure

UseJPEGImageDecoder()
UsePNGImageDecoder()
UseTIFFImageDecoder()
UseTGAImageDecoder()

Define SourceFilename.s
Define DestinationFilename.s
Define ShapeDetail.d
Define ColorDetail.d

SourceFilename = OpenFileRequester("Open", "", "Bitmap|*.bmp|JPEG|*.jpg;*.jpeg|PNG|*.png|All Formats|*.*", 0)

If SourceFilename = ""
  End
EndIf

Repeat
  ShapeDetail = ValD(InputRequester("Shape Detail", "Insert a value 0.0 (no shapes at all) < x <= 1.0 (detail) describing the shape detail here", "0.5"))
Until ShapeDetail > 0.0 And ShapeDetail <= 1.0

Repeat
  ColorDetail = ValD(InputRequester("Color Detail", "Insert a value 0.0 (no colors at all) < x <= 1.0 (full colors) describing the color detail here", StrD(5.0 / 255.0)))
Until ColorDetail > 0.0 And ColorDetail <= 1.0

Image = LoadImage(#PB_Any, SourceFilename)
If IsImage(Image)
  Vectorize(PathList(), ShapeDetail, ColorDetail, Image)
  
  PathImage = CreateImage(#PB_Any, ImageWidth(Image), ImageHeight(Image))
  If IsImage(PathImage)
    CreateImageFromPaths(PathList(), PathImage, 0.0, 0.0, 1.0, 1.0)
    DestinationFilename = SaveFileRequester("Save as SVG", "", "SVG|*.svg|All Files|*.*", 0)
    
    If SelectedFilePattern() = 0
      If Right(LCase(DestinationFilename), 4) <> ".svg"
        DestinationFilename + ".svg"
      EndIf
    EndIf
    
    CreateSVGFromPaths(DestinationFilename, PathList(), ImageWidth(Image), ImageHeight(Image), 0.0, 0.0, 1.0, 1.0)
    
    ViewImage(PathImage)
    FreeImage(PathImage)
  EndIf
  
  FreeImage(Image)
EndIf
; IDE Options = PureBasic 4.60 RC 1 (Windows - x64)
; CursorPosition = 12
; Folding = -
; EnableXP