; **********************************************************************
; *                                                                    *
; * Author   : Daniel Brall                                            *
; * Website  : http://www.bradan.eu/                                   *
; * E-Mail   : vectorizer@bradan.eu                                    *
; * Copyright: Daniel Brall                                            *
; *            All rights reserved!                                    *
; *                                                                    *
; * You are not allowed to modify and/or redistribute this sourcecode! *
; *                                                                    *
; **********************************************************************

XIncludeFile "vector.pbi"

Procedure FindPixel(image.i, color.i, *point.SVector)
  Protected x.i
  Protected y.i
  Protected result = #False
  
  StartDrawing(ImageOutput(image))
  DrawingMode(#PB_2DDrawing_AllChannels)
  For x = Int(*point\x) To ImageWidth(image) - 1
    For y = 0 To ImageHeight(image) - 1
      If Point(x, y) <> color
        *point\x = x
        *point\y = y
        result = #True
        Break 2
      EndIf
    Next y
  Next x
  StopDrawing()
  
  ProcedureReturn result
EndProcedure

Procedure PalettizeImage(image.i, colorSteps.i)
  Protected x.i
  Protected y.i
  Protected color.i
  Protected colorFactor.d
  Protected invColorFactor.d
  
  colorFactor = colorSteps / 255.0
  invColorFactor = 1.0 / colorFactor
  
  StartDrawing(ImageOutput(image))
  DrawingMode(#PB_2DDrawing_AllChannels)
  For x = 0 To ImageWidth(image) - 1
    For y = 0 To ImageHeight(image) - 1
      color = Point(x, y)
      color = RGBA(Round(Red(color) * colorFactor, #PB_Round_Nearest) * invColorFactor, Round(Green(color) * colorFactor, #PB_Round_Nearest) * invColorFactor, Round(Blue(color) * colorFactor, #PB_Round_Nearest) * invColorFactor, Round(Alpha(color) * colorFactor, #PB_Round_Nearest) * invColorFactor)
      Plot(x, y, color)
    Next y
  Next x
  StopDrawing()
EndProcedure

Procedure FloodCut(Array Area.i(2), image.i, *point.SVector, *min.SVector, *max.SVector)
  Protected NewList Stack.SVector()
  Protected color.i
  Protected currentPoint.SVector
  Protected x.i
  Protected y.i
  
  For x = *min\x To *max\x
    For y = *min\y To *max\y
      Area(x, y) = 0
    Next y
  Next x
  
  AddElement(Stack())
  Stack()\x = *point\x
  Stack()\y = *point\y
  
  CopyStructure(*point, *min, SVector)
  CopyStructure(*point, *max, SVector)
  
  right = ImageWidth(image) - 1
  bottom = ImageHeight(image) - 1
  
  StartDrawing(ImageOutput(image))
  DrawingMode(#PB_2DDrawing_AllChannels)
  color = Point(*point\x, *point\y)
  While ListSize(Stack()) > 0
    LastElement(Stack())
    CopyStructure(@Stack(), @currentPoint, SVector)
    DeleteElement(Stack(), 1)
    
    Area(Int(currentPoint\x) + 1, Int(currentPoint\y) + 1) = 1
    
    Plot(currentPoint\x, currentPoint\y, RGBA(0, 0, 0, 0))
    
    If currentPoint\x < *min\x
      *min\x = currentPoint\x
    EndIf
    If currentPoint\y < *min\y
      *min\y = currentPoint\y
    EndIf
    If currentPoint\x > *max\x
      *max\x = currentPoint\x
    EndIf
    If currentPoint\y > *max\y
      *max\y = currentPoint\y
    EndIf
    
    LastElement(Stack())
    If currentPoint\x < right And Point(currentPoint\x + 1, currentPoint\y) = color
      AddElement(Stack())
      Stack()\x = currentPoint\x + 1
      Stack()\y = currentPoint\y
    EndIf
    If currentPoint\y < bottom And Point(currentPoint\x, currentPoint\y + 1) = color
      AddElement(Stack())
      Stack()\x = currentPoint\x
      Stack()\y = currentPoint\y + 1
    EndIf
    If currentPoint\x > 0 And Point(currentPoint\x - 1, currentPoint\y) = color
      AddElement(Stack())
      Stack()\x = currentPoint\x - 1
      Stack()\y = currentPoint\y
    EndIf
    If currentPoint\y > 0 And Point(currentPoint\x, currentPoint\y - 1) = color
      AddElement(Stack())
      Stack()\x = currentPoint\x
      Stack()\y = currentPoint\y - 1
    EndIf
  Wend
  StopDrawing()
  
  ProcedureReturn color
EndProcedure
; IDE Options = PureBasic 4.51 (Windows - x64)
; CursorPosition = 66
; FirstLine = 48
; Folding = -
; EnableXP