; **********************************************************************
; *                                                                    *
; * Author   : Daniel Brall                                            *
; * Website  : http://www.bradan.eu/                                   *
; * E-Mail   : vectorizer@bradan.eu                                    *
; * Copyright: Daniel Brall                                            *
; *            All rights reserved!                                    *
; *                                                                    *
; * You are not allowed to modify and/or redistribute this sourcecode! *
; *                                                                    *
; **********************************************************************

Structure SVector
  x.d
  y.d
EndStructure

Procedure.d Dot(*a.SVector, *b.SVector)
  ProcedureReturn *a\x * *b\x + *a\y * *b\y
EndProcedure

Procedure.d Length(*a.SVector)
  ProcedureReturn Sqr(Dot(*a, *a))
EndProcedure

Procedure Mul(*a.SVector, factor.d)
  *a\x * factor
  *a\y * factor
EndProcedure

Procedure Add(*a.SVector, *b.SVector)
  *a\x + *b\x
  *a\y + *b\y
EndProcedure

Procedure Sub(*a.SVector, *b.SVector)
  *a\x - *b\x
  *a\y - *b\y
EndProcedure

Procedure.d Distance(*a.SVector, *b.SVector)
  Protected v.SVector
  
  CopyStructure(*a, @v, SVector)
  Sub(@v, *b)
  
  ProcedureReturn Length(@v)
EndProcedure

Procedure.d QuadDistance(*a.SVector, *b.SVector)
  Protected v.SVector
  
  CopyStructure(*a, @v, SVector)
  Sub(@v, *b)
  
  ProcedureReturn Dot(@v, @v)
EndProcedure

Procedure.d AngleBetween(*a.SVector, *b.SVector)
  Protected angle.d
  Protected denominator.d
  
  denominator = Length(*a) * Length(*b)
  
  If denominator > 0.0
    angle = ACos(Dot(*a, *b) / denominator)
  EndIf
  
  ProcedureReturn angle * 180.0 / #PI
EndProcedure
; IDE Options = PureBasic 4.51 (Windows - x64)
; CursorPosition = 65
; Folding = --
; EnableXP