; **********************************************************************
; *                                                                    *
; * Author   : Daniel Brall                                            *
; * Website  : http://www.bradan.eu/                                   *
; * E-Mail   : vectorizer@bradan.eu                                    *
; * Copyright: Daniel Brall                                            *
; *            All rights reserved!                                    *
; *                                                                    *
; * You are not allowed to modify and/or redistribute this sourcecode! *
; *                                                                    *
; **********************************************************************

XIncludeFile "math.pbi"
XIncludeFile "vector.pbi"
XIncludeFile "image.pbi"

Structure SLine
  timesUsed.i
  a.SVector
  b.SVector
EndStructure

Structure PLine
  *p.SLine
EndStructure

Structure SLineMap
  List Elements.PLine()
EndStructure

Procedure.d AbsD(value.d)
  If value < 0.0
    ProcedureReturn -1.0 * value
  EndIf
  
  ProcedureReturn value
EndProcedure

DisableDebugger
Procedure MarchingSquares(List Lines.SLine(), Map LineMap.SLineMap(), Array Area.i(2), *min.SVector, *max.SVector)
  Protected x.i
  Protected y.i
  
  ClearList(Lines())
  
  *min\x = MaxI(*min\x - 1, 0)
  *min\y = MaxI(*min\y - 1, 0)
  
  *max\x = MinI(*max\x + 1, ArraySize(Area(), 1) - 1)
  *max\y = MinI(*max\y + 1, ArraySize(Area(), 2) - 1)
  
  For x = *min\x To *max\x
    For y = *min\y To *max\y
      Select Area(x, y + 1) | (Area(x + 1, y + 1) << 1) | (Area(x + 1, y) << 2) | (Area(x, y) << 3)
;         Case %0000, %1111
        Case %0001, %1110
          AddElement(Lines())
          Lines()\timesUsed = 0
          
          Lines()\a\x    = x
          Lines()\a\y    = y + 0.5
          
          Lines()\b\x    = x + 0.5
          Lines()\b\y    = y + 1.0
          
          With LineMap(Str((Lines()\a\x + Lines()\a\y * ArraySize(Area(), 1)) * 2.0))
            AddElement(\Elements())
            \Elements()\p = @Lines()
          EndWith
          
          With LineMap(Str((Lines()\b\x + Lines()\b\y * ArraySize(Area(), 1)) * 2.0))
            AddElement(\Elements())
            \Elements()\p = @Lines()
          EndWith
          
        Case %0010, %1101
          AddElement(Lines())
          Lines()\timesUsed = 0
          
          Lines()\b\x    = x + 1.0
          Lines()\b\y    = y + 0.5
          
          Lines()\a\x    = x + 0.5
          Lines()\a\y    = y + 1.0
          
          With LineMap(Str((Lines()\a\x + Lines()\a\y * ArraySize(Area(), 1)) * 2.0))
            AddElement(\Elements())
            \Elements()\p = @Lines()
          EndWith
          
          With LineMap(Str((Lines()\b\x + Lines()\b\y * ArraySize(Area(), 1)) * 2.0))
            AddElement(\Elements())
            \Elements()\p = @Lines()
          EndWith
          
        Case %0011, %1100
          AddElement(Lines())
          Lines()\timesUsed = 0
          
          Lines()\a\x    = x
          Lines()\a\y    = y + 0.5
          
          Lines()\b\x    = x + 1.0
          Lines()\b\y    = y + 0.5
          
          With LineMap(Str((Lines()\a\x + Lines()\a\y * ArraySize(Area(), 1)) * 2.0))
            AddElement(\Elements())
            \Elements()\p = @Lines()
          EndWith
          
          With LineMap(Str((Lines()\b\x + Lines()\b\y * ArraySize(Area(), 1)) * 2.0))
            AddElement(\Elements())
            \Elements()\p = @Lines()
          EndWith
          
        Case %0100, %1011
          AddElement(Lines())
          Lines()\timesUsed = 0
          
          Lines()\a\x    = x + 0.5
          Lines()\a\y    = y
          
          Lines()\b\x    = x + 1.0
          Lines()\b\y    = y + 0.5
          
          With LineMap(Str((Lines()\a\x + Lines()\a\y * ArraySize(Area(), 1)) * 2.0))
            AddElement(\Elements())
            \Elements()\p = @Lines()
          EndWith
          
          With LineMap(Str((Lines()\b\x + Lines()\b\y * ArraySize(Area(), 1)) * 2.0))
            AddElement(\Elements())
            \Elements()\p = @Lines()
          EndWith
          
        Case %0101
          AddElement(Lines())
          Lines()\timesUsed = 0
          
          Lines()\a\x    = x + 0.5
          Lines()\a\y    = y
          
          Lines()\b\x    = x + 1.0
          Lines()\b\y    = y + 0.5
          
          With LineMap(Str((Lines()\a\x + Lines()\a\y * ArraySize(Area(), 1)) * 2.0))
            AddElement(\Elements())
            \Elements()\p = @Lines()
          EndWith
          
          With LineMap(Str((Lines()\b\x + Lines()\b\y * ArraySize(Area(), 1)) * 2.0))
            AddElement(\Elements())
            \Elements()\p = @Lines()
          EndWith
          
          AddElement(Lines())
          Lines()\timesUsed = 0
          
          Lines()\a\x    = x
          Lines()\a\y    = y + 0.5
          
          Lines()\b\x    = x + 0.5
          Lines()\b\y    = y + 1.0
          
          With LineMap(Str((Lines()\a\x + Lines()\a\y * ArraySize(Area(), 1)) * 2.0))
            AddElement(\Elements())
            \Elements()\p = @Lines()
          EndWith
          
          With LineMap(Str((Lines()\b\x + Lines()\b\y * ArraySize(Area(), 1)) * 2.0))
            AddElement(\Elements())
            \Elements()\p = @Lines()
          EndWith
          
        Case %1010
          AddElement(Lines())
          Lines()\timesUsed = 0
          
          Lines()\b\x    = x + 0.5
          Lines()\b\y    = y
          
          Lines()\a\x    = x
          Lines()\a\y    = y + 0.5
          
          With LineMap(Str((Lines()\a\x + Lines()\a\y * ArraySize(Area(), 1)) * 2.0))
            AddElement(\Elements())
            \Elements()\p = @Lines()
          EndWith
          
          With LineMap(Str((Lines()\b\x + Lines()\b\y * ArraySize(Area(), 1)) * 2.0))
            AddElement(\Elements())
            \Elements()\p = @Lines()
          EndWith
          
          AddElement(Lines())
          Lines()\timesUsed = 0
          
          Lines()\b\x    = x + 1.0
          Lines()\b\y    = y + 0.5
          
          Lines()\a\x    = x + 0.5
          Lines()\a\y    = y + 1.0
          
          With LineMap(Str((Lines()\a\x + Lines()\a\y * ArraySize(Area(), 1)) * 2.0))
            AddElement(\Elements())
            \Elements()\p = @Lines()
          EndWith
          
          With LineMap(Str((Lines()\b\x + Lines()\b\y * ArraySize(Area(), 1)) * 2.0))
            AddElement(\Elements())
            \Elements()\p = @Lines()
          EndWith
          
        Case %0110, %1001
          AddElement(Lines())
          Lines()\timesUsed = 0
          
          Lines()\a\x    = x + 0.5
          Lines()\a\y    = y
          
          Lines()\b\x    = x + 0.5
          Lines()\b\y    = y + 1.0
          
          With LineMap(Str((Lines()\a\x + Lines()\a\y * ArraySize(Area(), 1)) * 2.0))
            AddElement(\Elements())
            \Elements()\p = @Lines()
          EndWith
          
          With LineMap(Str((Lines()\b\x + Lines()\b\y * ArraySize(Area(), 1)) * 2.0))
            AddElement(\Elements())
            \Elements()\p = @Lines()
          EndWith
          
        Case %0111, %1000
          AddElement(Lines())
          Lines()\timesUsed = 0
          
          Lines()\b\x    = x + 0.5
          Lines()\b\y    = y
          
          Lines()\a\x    = x
          Lines()\a\y    = y + 0.5
          
          With LineMap(Str((Lines()\a\x + Lines()\a\y * ArraySize(Area(), 1)) * 2.0))
            AddElement(\Elements())
            \Elements()\p = @Lines()
          EndWith
          
          With LineMap(Str((Lines()\b\x + Lines()\b\y * ArraySize(Area(), 1)) * 2.0))
            AddElement(\Elements())
            \Elements()\p = @Lines()
          EndWith
      EndSelect
    Next y
  Next x
  
  ProcedureReturn 1
EndProcedure

Structure SPath
  avgColor.i
  List Path.SVector()
EndStructure

Procedure CreatePathFromLines(Width.i, List Lines.SLine(), Map LineMap.SLineMap(), List Paths.SPath(), color.i, scale.d)
  Protected *currentStartingLine.SLine
  Protected *currentElement.SLine
  Protected *nextElement.SLine
  Protected temp.SVector
  Protected minTimesUsed.i
  Protected mode.i
  Protected minMode.i
  
  If ListSize(Lines()) > 0
    FirstElement(Lines())
    *currentStartingLine = @Lines()
    
    AddElement(Paths())
    Paths()\avgColor = color
    
    ; connect all lines starting with this line
    *nextElement = *currentStartingLine
    
    Repeat
      *currentElement = *nextElement
      AddElement(Paths()\Path())
      
      CopyStructure(@*currentElement\a, @Paths()\Path(), SVector)
      Paths()\Path()\x = (Paths()\Path()\x - 0.5) * scale
      Paths()\Path()\y = (Paths()\Path()\y - 0.5) * scale
      
      ; find the next connected element with the minimum timesUsed
      *nextElement = #Null
      With LineMap(Str((*currentElement\b\x + *currentElement\b\y * Width) * 2.0))
        ForEach \Elements()
          If *currentElement <> \Elements()\p
            mode = 0
            If QuadDistance(@\Elements()\p\a, @*currentElement\b) < 0.001
              mode = 1
            ElseIf QuadDistance(@\Elements()\p\b, @*currentElement\b) < 0.001
              mode = 2
            EndIf
            
            If mode > 0 And (minTimesUsed >= \Elements()\p\timesUsed Or *nextElement = #Null)
              *nextElement = \Elements()\p
              minTimesUsed = \Elements()\p\timesUsed
              minMode      = mode
            EndIf
          EndIf
        Next
      EndWith
      
      ; now connect the two lines
      If *nextElement <> #Null
        If minMode = 2
          CopyStructure(@*nextElement\b, @temp, SVector)
          CopyStructure(@*nextElement\a, @*nextElement\b, SVector)
          CopyStructure(@temp, @*nextElement\a, SVector)
        EndIf
        
        *nextElement\timesUsed + 1
      EndIf
    Until *nextElement = *currentStartingLine Or *nextElement = #Null
    *currentStartingLine\timesUsed + 1
    
    If ListSize(Paths()\Path()) = 0
      DeleteElement(Paths())
    EndIf
  EndIf
EndProcedure
EnableDebugger

Procedure Vectorize(List PathList.SPath(), shapeDetail.d, colorDetail.d, image.i)
  Protected processedImage.i
  Protected point.SVector
  Protected color.i
  Protected NewList Lines.SLine()
  Protected NewMap LineMap.SLineMap()
  Protected min.SVector
  Protected max.SVector
  
  If shapeDetail = 0.0
    ProcedureReturn #False
  EndIf
  
  processedImage = CreateImage(#PB_Any, ImageWidth(image), ImageHeight(image), 32)
  StartDrawing(ImageOutput(processedImage))
  DrawingMode(#PB_2DDrawing_AllChannels)
  Box(0, 0, ImageWidth(processedImage), ImageHeight(processedImage), RGBA(0, 0, 0, 0))
  DrawImage(ImageID(image), 0, 0)
  StopDrawing()
  
  ResizeImage(processedImage, ImageWidth(processedImage) * shapeDetail, ImageHeight(processedImage) * shapeDetail)
  
  Protected Dim Area.i(ImageWidth(processedImage) + 2, ImageHeight(processedImage) + 2)
  
  If colorDetail < 0.0
    colorDetail = 0.0
  ElseIf colorDetail > 1.0
    colorDetail = 1.0
  EndIf
  
  PalettizeImage(processedImage, colorDetail * 255.0)
  
  shapeDetail = 1.0 / shapeDetail
  
  For x = 0 To ArraySize(Area(), 1)
    For y = 0 To ArraySize(Area(), 2)
      Area(x, y) = 0
    Next y
  Next x
  
  While FindPixel(processedImage, RGBA(0, 0, 0, 0), @point)
    LastElement(PathList())
    color = FloodCut(Area(), processedImage, @point, @min, @max)
    
    MarchingSquares(Lines(), LineMap(), Area(), @min, @max)
    CreatePathFromLines(ArraySize(Area(), 1), Lines(), LineMap(), PathList(), color, shapeDetail)
    
    ClearList(Lines())
    ClearMap(LineMap())
  Wend
  
  FreeImage(processedImage)
  
  ProcedureReturn #True
EndProcedure
; IDE Options = PureBasic 4.51 (Windows - x64)
; CursorPosition = 306
; FirstLine = 263
; Folding = -
; EnableXP